# Generated by Django 2.2rc1 on 2019-03-30 12:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("projects", "0007_auto_20190321_1449")]

    operations = [
        migrations.AddField(
            model_name="service",
            name="is_logging_prohibited",
            field=models.BooleanField(
                default=False,
                help_text="Use this for service entries which are only used for budgeting.",
                verbose_name="is logging prohibited",
            ),
        )
    ]
