# Generated by Django 2.2.3 on 2019-07-15 15:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("contacts", "0008_auto_20190703_0801")]

    operations = [
        migrations.AddField(
            model_name="organization",
            name="is_private_person",
            field=models.BooleanField(default=False, verbose_name="is private person"),
        )
    ]
