# Generated by Django 3.0 on 2019-12-07 12:04

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("awt", "0004_auto_20190327_0945"),
    ]

    operations = [
        migrations.CreateModel(
            name="WorkingTimeModel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100, verbose_name="name")),
            ],
            options={
                "verbose_name": "working time model",
                "verbose_name_plural": "working time models",
                "ordering": ["name"],
            },
        ),
        migrations.AddField(
            model_name="year",
            name="working_time_model",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="awt.WorkingTimeModel",
                verbose_name="working time model",
            ),
        ),
    ]
