# Generated by Django 2.1.7 on 2019-03-21 13:49

from django.db import migrations

from workbench.tools import search


class Migration(migrations.Migration):

    dependencies = [("offers", "0003_auto_20190304_2247")]

    operations = [
        migrations.RunSQL(search.drop_old_shit("offers_offer")),
        migrations.RunSQL(search.create_structure("offers_offer")),
        migrations.RunSQL(
            search.fts("offers_offer", ["title", "description", "postal_address"])
        ),
    ]
