# Generated by Django 2.2.3 on 2019-07-19 11:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("expenses", "0002_expensereport_closed_on")]

    operations = [
        migrations.RunSQL("SELECT audit_audit_table('expenses_expensereport')", "")
    ]
