# Generated by Django 3.1.2 on 2020-10-17 08:18

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("accounts", "0010_auto_20201016_0957"),
        ("audit", "0004_auto_20201017_1016"),
    ]

    operations = [
        migrations.RunSQL(
            "SELECT audit_audit_table_initial('accounts_team');",
            "DELETE FROM audit_logged_actions WHERE table_name='accounts_team';",
        ),
    ]
