# Generated by Django 3.1a1 on 2020-05-25 10:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("offers", "0008_auto_20190916_1110"),
    ]

    operations = [
        migrations.AddField(
            model_name="offer",
            name="valid_until",
            field=models.DateField(blank=True, null=True, verbose_name="valid until"),
        ),
        migrations.RunSQL(
            "UPDATE offers_offer SET valid_until=offered_on + interval '59 days'"
        ),
    ]
