# Generated by Django 3.0 on 2019-12-07 12:04

from django.db import migrations


def forwards(apps, schema_editor):
    wtm = apps.get_model("awt", "WorkingTimeModel").objects.create(name="Zürich")
    apps.get_model("awt", "Year").objects.update(working_time_model=wtm.id)
    apps.get_model("accounts", "User").objects.update(working_time_model=wtm.id)


class Migration(migrations.Migration):

    dependencies = [
        ("awt", "0005_auto_20191207_1304"),
        ("accounts", "0005_user_working_time_model"),
    ]

    operations = [migrations.RunPython(forwards)]
