# Generated by Django 2.2.1 on 2019-05-18 12:25

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("logbook", "0006_auto_20190321_1448"),
    ]

    operations = [
        migrations.AddField(
            model_name="loggedcost",
            name="are_expenses",
            field=models.BooleanField(
                default=False, verbose_name="paid from my own pocket"
            ),
        ),
        migrations.AddField(
            model_name="loggedcost",
            name="expenses_reimbursed_at",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="expenses reimbursed at"
            ),
        ),
        migrations.AddField(
            model_name="loggedcost",
            name="rendered_by",
            field=models.ForeignKey(
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="loggedcosts",
                to=settings.AUTH_USER_MODEL,
                verbose_name="rendered by",
            ),
            preserve_default=False,
        ),
    ]
