# Generated by Django 2.2rc1 on 2019-03-24 17:15

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("credit_control", "0004_auto_20190324_1813")]

    operations = [
        migrations.AlterField(
            model_name="creditentry",
            name="ledger",
            field=models.ForeignKey(
                default=None,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="transactions",
                to="credit_control.Ledger",
                verbose_name="ledger",
            ),
            preserve_default=False,
        )
    ]
