# Generated by Django 2.2.4 on 2019-08-18 13:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("projects", "0013_auto_20190630_2219")]

    operations = [
        migrations.RunSQL(
            "UPDATE projects_project SET type='order' WHERE type='acquisition'"
        ),
        migrations.AlterField(
            model_name="project",
            name="type",
            field=models.CharField(
                choices=[
                    ("order", "Order"),
                    ("maintenance", "Maintenance"),
                    ("internal", "Internal"),
                ],
                max_length=20,
                verbose_name="type",
            ),
        ),
    ]
