# Generated by Django 3.0 on 2019-12-10 14:54

import django.db.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("expenses", "0003_auto_20190719_1354"),
    ]

    operations = [
        migrations.CreateModel(
            name="ExchangeRates",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("day", models.DateField(unique=True, verbose_name="day")),
                ("rates", django.db.models.JSONField()),
            ],
            options={
                "verbose_name": "exchange rate",
                "verbose_name_plural": "exchange rates",
                "ordering": ["-day"],
                "get_latest_by": "day",
            },
        ),
    ]
