# Generated by Django 3.1.2 on 2020-10-16 07:57

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("accounts", "0009_user_person"),
    ]

    operations = [
        migrations.RunSQL(
            """
SELECT audit_audit_table('accounts_team');
UPDATE accounts_team SET id=id;
                """,
            "",
        ),
    ]
