# Generated by Django 2.2.2 on 2019-06-22 08:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("credit_control", "0005_auto_20190324_1815")]

    operations = [
        migrations.AddField(
            model_name="ledger",
            name="parser",
            field=models.CharField(
                choices=[
                    ("zkb-csv", "ZKB CSV"),
                    ("postfinance-csv", "PostFinance CSV"),
                ],
                default="zkb-csv",
                max_length=20,
                verbose_name="parser",
            ),
            preserve_default=False,
        )
    ]
