# Generated by Django 2.2.2 on 2019-07-03 06:01

from django.db import migrations, models

from workbench.tools import search


class Migration(migrations.Migration):

    dependencies = [("contacts", "0006_auto_20190321_1445")]

    operations = [
        migrations.AddField(
            model_name="person",
            name="_fts",
            field=models.TextField(blank=True, editable=False),
        ),
        migrations.RunSQL(
            search.fts(
                "contacts_person",
                ["given_name", "family_name", "address", "notes", "_fts"],
            )
        ),
    ]
