# Generated by Django 2.2.3 on 2019-07-19 07:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("logbook", "0010_auto_20190712_1324"),
        ("logbook", "0010_auto_20190709_2140"),
    ]

    operations = []
