# Generated by Django 3.0.4 on 2020-03-10 18:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("reporting", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="CostCenter",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=200, verbose_name="title")),
                (
                    "position",
                    models.PositiveIntegerField(default=0, verbose_name="position"),
                ),
            ],
            options={
                "verbose_name": "cost center",
                "verbose_name_plural": "cost centers",
                "ordering": ("position", "id"),
            },
        ),
        migrations.RunSQL(
            "SELECT audit_audit_table('reporting_costcenter', ARRAY['position']);"
        ),
    ]
