# Generated by Django 2.2.1 on 2019-05-18 12:43

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("logbook", "0007_auto_20190518_1425")]

    operations = [
        migrations.RunSQL("UPDATE logbook_loggedcost SET rendered_by_id=created_by_id")
    ]
