# Generated by Django 2.1.7 on 2019-03-12 11:54

from decimal import Decimal

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("logbook", "0004_auto_20190310_1134")]

    operations = [
        migrations.AlterField(
            model_name="loggedhours",
            name="hours",
            field=models.DecimalField(
                decimal_places=1,
                max_digits=10,
                validators=[django.core.validators.MinValueValidator(Decimal("0.1"))],
                verbose_name="hours",
            ),
        )
    ]
