# Generated by Django 3.0 on 2019-12-07 12:04

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("awt", "0005_auto_20191207_1304"),
        ("accounts", "0004_user_language"),
    ]

    operations = [
        migrations.AddField(
            model_name="user",
            name="working_time_model",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="awt.WorkingTimeModel",
                verbose_name="working time model",
            ),
        ),
    ]
