# Generated by Django 3.1.2 on 2020-10-17 08:16

import io
import os

from django.conf import settings
from django.db import migrations


with io.open(os.path.join(settings.BASE_DIR, "stuff", "audit.sql")) as f:
    AUDIT_SQL = f.read()


class Migration(migrations.Migration):

    dependencies = [
        ("audit", "0003_auto_20190703_0813"),
    ]

    operations = [
        migrations.RunSQL(AUDIT_SQL),
    ]
