# Generated by Django 3.0.1 on 2020-01-04 08:38

from django.db import migrations, models

from workbench.tools import search


class Migration(migrations.Migration):

    dependencies = [
        ("invoices", "0019_auto_20191002_1005"),
        ("credit_control", "0006_ledger_parser"),
    ]

    operations = [
        migrations.AddField(
            model_name="creditentry",
            name="_fts",
            field=models.TextField(blank=True, editable=False),
        ),
        migrations.RunSQL(
            search.fts(
                "credit_control_creditentry",
                ["reference_number", "total", "payment_notice", "notes", "_fts"],
            )
        ),
    ]
