# Generated by Django 2.2.4 on 2019-08-16 15:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("contacts", "0009_organization_is_private_person")]

    operations = [
        migrations.AddField(
            model_name="organization",
            name="default_billing_address",
            field=models.TextField(
                blank=True,
                help_text="Mainly useful for organizations with a centralized accounts payable departement.",
                verbose_name="default billing address",
            ),
        )
    ]
