# Generated by Django 3.0.6 on 2020-05-11 11:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("logbook", "0018_auto_20200511_1324"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="break",
            options={
                "ordering": ["-starts_at"],
                "verbose_name": "break",
                "verbose_name_plural": "breaks",
            },
        ),
        migrations.RemoveField(
            model_name="break",
            name="ends_at",
        ),
        migrations.RemoveField(
            model_name="break",
            name="starts_at",
        ),
        migrations.RemoveField(
            model_name="break",
            name="day",
        ),
        migrations.RenameField("break", "_ends_at", "ends_at"),
        migrations.RenameField("break", "_starts_at", "starts_at"),
        migrations.AlterField(
            model_name="break",
            name="ends_at",
            field=models.DateTimeField(null=True, verbose_name="ends at"),
        ),
        migrations.AlterField(
            model_name="break",
            name="starts_at",
            field=models.DateTimeField(null=True, verbose_name="starts at"),
        ),
    ]
