# Generated by Django 2.1.7 on 2019-03-21 13:49

from django.db import migrations

from workbench.tools import search


class Migration(migrations.Migration):

    dependencies = [("projects", "0006_auto_20190312_1254")]

    operations = [
        migrations.RunSQL(search.drop_old_shit("projects_project")),
        migrations.RunSQL(search.create_structure("projects_project")),
        migrations.RunSQL(search.fts("projects_project", ["title", "description"])),
    ]
