# Generated by Django 2.1.7 on 2019-03-12 08:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("contacts", "0003_auto_20190305_0809")]

    operations = [
        migrations.AddField(
            model_name="person",
            name="is_archived",
            field=models.BooleanField(
                default=False,
                verbose_name="is archived",
                help_text="It is preferrable to archive records instead of deleting them.",
            ),
        )
    ]
