# Generated by Django 2.2.5 on 2019-09-15 10:58

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("logbook", "0014_auto_20190915_1250")]

    operations = [migrations.RemoveField(model_name="loggedcost", name="project")]
