# Generated by Django 2.1.7 on 2019-03-21 13:47

from django.db import migrations

from workbench.tools import search


class Migration(migrations.Migration):

    dependencies = [("credit_control", "0002_auto_20190308_1752")]

    operations = [
        migrations.RunSQL(search.drop_old_shit("credit_control_creditentry")),
        migrations.RunSQL(search.create_structure("credit_control_creditentry")),
        migrations.RunSQL(
            search.fts(
                "credit_control_creditentry",
                ["reference_number", "total", "payment_notice", "notes"],
            )
        ),
    ]
