# Generated by Django 2.2.2 on 2019-06-30 20:13

from django.db import migrations, models

from workbench.tools import search


class Migration(migrations.Migration):

    dependencies = [("projects", "0011_auto_20190629_0943")]

    operations = [
        migrations.AddField(
            model_name="project",
            name="_fts",
            field=models.TextField(blank=True, editable=False),
        ),
        migrations.RunSQL(
            search.fts("projects_project", ["title", "description", "_fts"])
        ),
    ]
