# Generated by Django 2.2.2 on 2019-07-09 19:40

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("expenses", "0001_initial"),
        ("logbook", "0009_auto_20190628_1659"),
    ]

    operations = [
        migrations.RemoveField(model_name="loggedcost", name="expenses_reimbursed_at"),
        migrations.AddField(
            model_name="loggedcost",
            name="expense_report",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="expenses",
                to="expenses.ExpenseReport",
                verbose_name="expense report",
            ),
        ),
    ]
