# Generated by Django 3.0 on 2019-12-07 13:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("awt", "0007_auto_20191207_1307"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="workingtimemodel",
            options={
                "ordering": ["position", "id"],
                "verbose_name": "working time model",
                "verbose_name_plural": "working time models",
            },
        ),
        migrations.AddField(
            model_name="workingtimemodel",
            name="position",
            field=models.IntegerField(default=0, verbose_name="position"),
        ),
    ]
