# Generated by Django 2.2.3 on 2019-07-21 08:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("accounts", "0003_user_enforce_same_week_logging")]

    operations = [
        migrations.AddField(
            model_name="user",
            name="language",
            field=models.CharField(
                choices=[("en", "english"), ("de", "german")],
                max_length=10,
                verbose_name="language",
            ),
        )
    ]
