# Generated by Django 3.1b1 on 2020-07-10 15:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("contacts", "0012_person_date_of_birth"),
    ]

    operations = [
        migrations.AddField(
            model_name="organization",
            name="is_archived",
            field=models.BooleanField(
                default=False,
                help_text="It is preferrable to archive records instead of deleting them.",
                verbose_name="is archived",
            ),
        ),
    ]
