# Generated by Django 2.1.7 on 2019-03-12 11:54

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("invoices", "0007_auto_20190311_1456")]

    operations = [
        migrations.AlterField(
            model_name="service",
            name="effort_hours",
            field=models.DecimalField(
                blank=True,
                decimal_places=1,
                max_digits=10,
                null=True,
                verbose_name="hours",
            ),
        ),
        migrations.AlterField(
            model_name="service",
            name="service_hours",
            field=models.DecimalField(
                decimal_places=1, default=0, max_digits=10, verbose_name="service hours"
            ),
        ),
    ]
