# Generated by Django 2.2 on 2019-06-28 14:59

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("logbook", "0008_auto_20190518_1443")]

    operations = [
        migrations.AlterField(
            model_name="loggedhours",
            name="created_at",
            field=models.DateTimeField(
                db_index=True,
                default=django.utils.timezone.now,
                verbose_name="created at",
            ),
        )
    ]
