# Generated by Django 3.0.5 on 2020-05-04 11:38

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("timer", "0005_timestamp_project"),
    ]

    operations = [
        migrations.DeleteModel(
            name="TimerState",
        ),
    ]
