# Generated by Django 3.0 on 2019-12-10 14:37

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("logbook", "0015_remove_loggedcost_project"),
    ]

    operations = [
        migrations.AddField(
            model_name="loggedcost",
            name="expense_cost",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=10,
                null=True,
                validators=[django.core.validators.MinValueValidator(0)],
                verbose_name="original cost",
            ),
        ),
        migrations.AddField(
            model_name="loggedcost",
            name="expense_currency",
            field=models.CharField(
                blank=True, max_length=3, verbose_name="original currency"
            ),
        ),
    ]
