# Generated by Django 3.0.5 on 2020-04-23 09:22

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("projects", "0018_auto_20200402_2212"),
        ("timer", "0004_timestamp_logged_break"),
    ]

    operations = [
        migrations.AddField(
            model_name="timestamp",
            name="project",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="projects.Project",
                verbose_name="project",
            ),
        ),
    ]
