# Generated by Django 3.0.2 on 2020-01-30 13:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("invoices", "0019_auto_20191002_1005"),
    ]

    operations = [
        migrations.AddField(
            model_name="recurringinvoice",
            name="create_invoice_on_day",
            field=models.IntegerField(
                default=-20,
                help_text="Invoices are created 20 days before their period begins by default.",
                verbose_name="create invoice on day",
            ),
        ),
    ]
