# Generated by Django 3.1.5 on 2021-01-11 08:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("circles", "0002_auto_20190916_0843"),
    ]

    operations = [
        migrations.AddField(
            model_name="role",
            name="work_category",
            field=models.CharField(
                blank=True,
                choices=[
                    ("paid-work", "Paid work"),
                    ("knowledge-transfer", "Knowledge transfer"),
                    ("social-care", "Social care"),
                    ("outreach", "Outreach"),
                    ("other", "Other internal matters"),
                ],
                max_length=20,
                verbose_name="category",
            ),
        ),
    ]
