# Generated by Django 2.2.3 on 2019-07-19 09:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("expenses", "0001_initial")]

    operations = [
        migrations.AddField(
            model_name="expensereport",
            name="closed_on",
            field=models.DateField(blank=True, null=True, verbose_name="closed on"),
        )
    ]
