# Generated by Django 3.1.2 on 2020-10-22 06:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("planning", "0003_delete_planningteammembership"),
    ]

    operations = [
        migrations.AddField(
            model_name="planningrequest",
            name="is_provisional",
            field=models.BooleanField(default=False, verbose_name="is provisional"),
        ),
    ]
