# Generated by Django 2.1.7 on 2019-03-07 08:25

from decimal import Decimal

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("invoices", "0004_recurringinvoice")]

    operations = [
        migrations.AddField(
            model_name="invoice",
            name="third_party_costs",
            field=models.DecimalField(
                decimal_places=2,
                default=Decimal("0.00"),
                help_text="Only used for statistical purposes.",
                max_digits=10,
                validators=[django.core.validators.MinValueValidator(0)],
                verbose_name="third party costs",
            ),
        ),
        migrations.AddField(
            model_name="recurringinvoice",
            name="third_party_costs",
            field=models.DecimalField(
                decimal_places=2,
                default=Decimal("0.00"),
                help_text="Only used for statistical purposes.",
                max_digits=10,
                validators=[django.core.validators.MinValueValidator(0)],
                verbose_name="third party costs",
            ),
        ),
    ]
