# Generated by Django 3.0.5 on 2020-05-06 19:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("timer", "0006_delete_timerstate"),
    ]

    operations = [
        migrations.AlterField(
            model_name="timestamp",
            name="type",
            field=models.CharField(
                choices=[
                    ("start", "Start"),
                    ("stop", "Stop"),
                    ("logbook", "Logbook"),
                    ("break", "Break"),
                ],
                max_length=10,
                verbose_name="type",
            ),
        ),
        migrations.RunSQL(
            "UPDATE timer_timestamp SET type='stop' WHERE type='split';", ""
        ),
    ]
