# Generated by Django 3.0.4 on 2020-03-24 07:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("invoices", "0020_recurringinvoice_create_invoice_on_day"),
    ]

    operations = [
        migrations.AddField(
            model_name="invoice",
            name="service_period_from",
            field=models.DateField(
                blank=True, null=True, verbose_name="service period from"
            ),
        ),
        migrations.AddField(
            model_name="invoice",
            name="service_period_until",
            field=models.DateField(
                blank=True, null=True, verbose_name="service period until"
            ),
        ),
    ]
