# Generated by Django 3.0.4 on 2020-03-10 18:58

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("reporting", "0002_costcenter"),
        ("projects", "0016_auto_20190915_1249"),
    ]

    operations = [
        migrations.AddField(
            model_name="project",
            name="cost_center",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="projects",
                to="reporting.CostCenter",
                verbose_name="cost center",
            ),
        ),
    ]
