# Generated by Django 3.0 on 2019-12-10 09:48

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("awt", "0008_auto_20191207_1412"),
    ]

    operations = [
        migrations.AddField(
            model_name="employment",
            name="green_hours_target",
            field=models.SmallIntegerField(
                blank=True, null=True, verbose_name="green hours target"
            ),
        ),
        migrations.AddField(
            model_name="employment",
            name="hourly_labor_costs",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=10,
                null=True,
                validators=[django.core.validators.MinValueValidator(0)],
                verbose_name="hourly labor costs",
            ),
        ),
        migrations.AlterField(
            model_name="employment",
            name="notes",
            field=models.CharField(blank=True, max_length=500, verbose_name="notes"),
        ),
    ]
