# Generated by Django 2.2 on 2019-04-04 09:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("invoices", "0010_auto_20190321_1448")]

    operations = [
        migrations.AddField(
            model_name="invoice",
            name="last_reminded_on",
            field=models.DateField(
                blank=True, null=True, verbose_name="last reminded on"
            ),
        ),
        migrations.AlterField(
            model_name="invoice",
            name="status",
            field=models.PositiveIntegerField(
                choices=[
                    (10, "In preparation"),
                    (20, "Sent"),
                    (40, "Paid"),
                    (50, "Canceled"),
                ],
                default=10,
                verbose_name="status",
            ),
        ),
    ]
