# Generated by Django 2.2.2 on 2019-06-29 07:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("projects", "0010_service_role")]

    operations = [
        migrations.RemoveField(model_name="service", name="is_logging_prohibited"),
        migrations.AddField(
            model_name="service",
            name="allow_logging",
            field=models.BooleanField(
                default=True,
                help_text="Deactivate this for service entries which are only used for budgeting.",
                verbose_name="allow logging",
            ),
        ),
    ]
