# Generated by Django 3.0.2 on 2020-01-11 13:34

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Accruals",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "cutoff_date",
                    models.DateField(unique=True, verbose_name="cutoff date"),
                ),
                (
                    "accruals",
                    models.DecimalField(
                        decimal_places=2,
                        max_digits=10,
                        validators=[django.core.validators.MinValueValidator(0)],
                        verbose_name="accruals",
                    ),
                ),
            ],
            options={
                "verbose_name": "accruals",
                "verbose_name_plural": "accruals",
                "ordering": ["-cutoff_date"],
            },
        ),
    ]
