# Generated by Django 2.2.5 on 2019-09-16 06:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("circles", "0001_initial")]

    operations = [
        migrations.AlterModelOptions(
            name="role",
            options={
                "ordering": ["is_removed", "-for_circle", "name"],
                "verbose_name": "role",
                "verbose_name_plural": "roles",
            },
        ),
        migrations.AddField(
            model_name="role",
            name="for_circle",
            field=models.BooleanField(default=False, verbose_name="for the circle"),
        ),
    ]
