# Generated by Django 3.0.4 on 2020-03-28 19:31

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("logbook", "0017_break"),
        ("timer", "0003_timestamp_logged_hours"),
    ]

    operations = [
        migrations.AddField(
            model_name="timestamp",
            name="logged_break",
            field=models.OneToOneField(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="logbook.Break",
                verbose_name="break",
            ),
        ),
    ]
