# Generated by Django 2.2.5 on 2019-09-10 11:05

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("projects", "0014_auto_20190818_1529")]

    operations = [
        migrations.AddField(
            model_name="project",
            name="flat_rate",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="Set this if you want all services to have the same hourly rate.",
                max_digits=10,
                null=True,
                validators=[django.core.validators.MinValueValidator(0)],
                verbose_name="flat rate",
            ),
        )
    ]
