# Generated by Django 3.0 on 2019-12-07 12:07

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("awt", "0007_auto_20191207_1307"),
        ("accounts", "0005_user_working_time_model"),
    ]

    operations = [
        migrations.AlterField(
            model_name="user",
            name="working_time_model",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="awt.WorkingTimeModel",
                verbose_name="working time model",
            ),
        ),
    ]
