# Generated by Django 2.1.7 on 2019-03-12 11:54

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("awt", "0002_auto_20190304_2247")]

    operations = [
        migrations.AlterField(
            model_name="year",
            name="working_time_per_day",
            field=models.DecimalField(
                decimal_places=1,
                max_digits=10,
                validators=[django.core.validators.MinValueValidator(0)],
                verbose_name="working time per day",
            ),
        )
    ]
