# Generated by Django 2.1.7 on 2019-03-21 07:33

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("invoices", "0008_auto_20190312_1254")]

    operations = [
        migrations.RunSQL("SELECT audit_audit_table('invoices_service');", "")
    ]
