# Generated by Django 2.2.3 on 2019-07-12 11:24

import datetime

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("logbook", "0009_auto_20190628_1659")]

    operations = [
        migrations.AlterField(
            model_name="loggedcost",
            name="rendered_on",
            field=models.DateField(
                db_index=True, default=datetime.date.today, verbose_name="rendered on"
            ),
        ),
        migrations.AlterField(
            model_name="loggedhours",
            name="rendered_on",
            field=models.DateField(
                db_index=True, default=datetime.date.today, verbose_name="rendered on"
            ),
        ),
    ]
