# Generated by Django 3.0.2 on 2020-01-14 06:57

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("accounts", "0007_team"),
    ]

    operations = [
        migrations.RunSQL(
            "SELECT audit_audit_table('accounts_user', ARRAY['password', 'last_login']);"
        ),
        migrations.RunSQL(
            "UPDATE audit_logged_actions SET"
            " row_data=delete(delete(row_data, 'password'), 'last_login'),"
            " changed_fields=delete(delete(changed_fields, 'password'), 'last_login')"
            " WHERE table_name='accounts_user';"
        ),
    ]
