# Generated by Django 2.2.5 on 2019-09-16 09:05

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("offers", "0005_offer_show_service_details")]

    operations = [
        migrations.AddField(
            model_name="offer",
            name="total_excl_tax",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=10,
                null=True,
                validators=[django.core.validators.MinValueValidator(0)],
                verbose_name="total excl. tax",
            ),
        )
    ]
