# Generated by Django 3.0.3 on 2020-02-19 11:34

from django.conf import settings
from django.db import migrations

from workbench.tools import search


class Migration(migrations.Migration):

    dependencies = [
        ("deals", "0001_initial"),
    ]

    operations = [
        migrations.RunSQL(search.drop_old_shit("deals_deal")),
        migrations.RunSQL(search.create_structure("deals_deal")),
        migrations.RunSQL(
            search.fts(
                "deals_deal",
                ["title", "description", "closing_notice", "_fts"],
            )
        ),
        migrations.RunSQL(
            "SELECT audit_audit_table('deals_valuetype');"
            "SELECT audit_audit_table('deals_value');"
            "SELECT audit_audit_table('deals_deal', ARRAY['_fts', 'fts_document']);"
        ),
    ]
