# Generated by Django 2.2 on 2019-04-12 15:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("projects", "0008_service_is_logging_prohibited")]

    operations = [
        migrations.AddField(
            model_name="service",
            name="is_optional",
            field=models.BooleanField(
                default=False,
                help_text="Optional services to not count towards the offer total.",
                verbose_name="is optional",
            ),
        )
    ]
