# Generated by Django 2.1.7 on 2019-03-08 16:52

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("credit_control", "0001_initial")]

    operations = [
        migrations.RunSQL("SELECT audit_audit_table('credit_control_creditentry');", "")
    ]
