# Generated by Django 2.2.5 on 2019-09-16 09:08

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("invoices", "0016_auto_20190916_1108")]

    operations = [
        migrations.RunSQL(
            "SET SESSION application_name TO 'Migrations';"
            "UPDATE invoices_invoice SET total_excl_tax=subtotal - discount - down_payment_total;"
            "UPDATE invoices_recurringinvoice SET total_excl_tax=subtotal - discount;"
        )
    ]
